% Use dense correspondence to fit spline model to tune color
% input, im1_idx, im2_idx

%% Set paramters
%{
saturation_preproc=0;
if saturation_preproc
	%% Fit color saturation
	hsv_im1 = rgb2hsv(im1); hsv_im2 = rgb2hsv(im2);
	col_hsv_im1 = reshape(hsv_im1, [size(im1,1)*size(im1,2) size(im1,3)]);
	col_hsv_im2 = reshape(hsv_im2, [size(im2,1)*size(im2,2) size(im2,3)]);
	s1 = col_hsv_im1(im1_idx, 2);
	s2 = col_hsv_im2(im2_idx, 2);
	
	%% Tune saturation of image
	hsv_im1(:,:,2) = hsv_im1(:,:,2) * (sum(s2) / sum(s1));
	sat_im1 = hsv2rgb(hsv_im1); sat_im1 = im2uint8(sat_im1);
	
	% the preprocessed version of im1
	proc_im1 = sat_im1;
else
	proc_im1 = im1;
end
%}

% figure; imshow(im1);
% figure; imshow(sat_im1);

%% Start with r-channel
% when compute color correlation, x axis from im1, y axis from im2

function tuned_im1=flowenhance_spline(im1, im2, im1_idx, im2_idx)
	colorspace='rgb';
	switch colorspace
	case 'rgb'
		% do nothing
		preproc_im1=im1; % preprocessed
		preproc_im2=im2;
	case 'lab'
		colorTransform = makecform('srgb2lab');
		preproc_im1 = applycform(im1, colorTransform);
		preproc_im2 = applycform(im2, colorTransform);
	end

	col_im1 = reshape(preproc_im1, [size(preproc_im1,1)*size(preproc_im1,2) size(preproc_im1,3)]);
	col_im2 = reshape(preproc_im2, [size(preproc_im2,1)*size(preproc_im2,2) size(preproc_im2,3)]);
	pixels1 = col_im1(im1_idx, :);
	pixels2 = col_im2(im2_idx, :);
	[channel_model, fitted_splines] = fit_global_parametric(pixels1, pixels2);

	%% Apply color transform
	tuned_im1 = im1;
	for i_channel = 1:3
		xp = 1:256;
		yp = slmeval(xp,channel_model(i_channel).model,0); % use yp as color mapping
		
		% limit mapping range
		yp(yp<1) = 1; yp(yp>256)=256;
		tuned_im1(:,:,i_channel) = yp(preproc_im1(:,:,i_channel)+1);
		tuned_im1(:,:,i_channel) = tuned_im1(:,:,i_channel)-1;
	end

	switch colorspace
	case 'rgb'
		% do nothing
	case 'lab'
		colorTransform = makecform('lab2srgb');
		tuned_im1=applycform(tuned_im1, colorTransform);
	end
end
