function gtFlow = load_gtflow(gtFilename, srcScale, tgtScale)
    % LOAD_GTFLOW load ground truth flow
    gtFlow = readFlowFile(gtFilename);
    thres = 1e8; % larger than this is unknown
    gtFlow(abs(gtFlow)>thres) = 0;
    
    if (nargin == 3)
    % Scale ground truth
        gtFlow = scale_gt(gtFlow, srcScale, tgtScale);
    end
end


function scaledGtFlow = scale_gt(gtFlow, srcScale, tgtScale)
    % SCALE_GT 
    % scale the ground truth flow based on the src and target scale
    %{
    source is src_ratio
    target is tgt_ratio

    x1 + u -> x2  => src_ratio * x1 + ?? -> tgt_ratio * x2
    src * x1 + (1-src) * x1 + u -> tgt * x2 + (1-tgt) * x2
    src * x1 + ((1-src) * x1 - (1-tgt) * x2 + u) -> tgt * x2

    (1-src) * x1 - (1-tgt) * x2 + u
    x2 = x1 + u
    (1-src) * x1 - (1-tgt) * (x1 + u) + u
    (-src+tgt) * x1 + tgt*u

    example, src = 0.7, tgt = 0.2, => -0.5*x1 + 0.2*u
    %}

    %{
    vx = gt_flo(:,:,1); vy = gt_flo(:,:,2);
    [X, Y] = meshgrid(1:size(vx,2), 1:size(vx,1));
    new_size = size(imresize(vx, src_ratio));
    [X1, Y1] = meshgrid(1:new_size(2), 1:new_size(1));
    X1 = X1 ./ src_ratio; Y1 = Y1 ./ src_ratio;
    vx = (-src_ratio+tgt_ratio) * X + tgt_ratio * vx; % how to calculate this formulation
    vy = (-src_ratio+tgt_ratio) * Y + tgt_ratio * vy;

    vx = interp2(X, Y, vx, X1, Y1); vx(isnan(vx)) = 0;
    vy = interp2(X, Y, vy, X1, Y1); vy(isnan(vy)) = 0;
    scaled_gt_flo(:,:,1) = vx; scaled_gt_flo(:,:,2) = vy;
    %}
    vx = gtFlow(:,:,1); vy = gtFlow(:,:,2);
    
    [h, w, c] = size(gtFlow);
    
    [XX, YY] = meshgrid(1:w, 1:h);
    XX1 = XX + vx; YY1 = YY + vy;
    XX2 = XX1 .* tgtScale; YY2 = YY1 .* tgtScale;
    XX2 = imresize(XX2, srcScale); YY2 = imresize(YY2, srcScale);
    [XX3, YY3] = meshgrid(1:round(w*srcScale), 1:round(h*srcScale));
    scaledGtFlow(:,:,1) = XX2 - XX3; scaledGtFlow(:,:,2) = YY2 - YY3;
end