% Compute qualitative result
% Setup code
clearvars;
addpath('include/flow-code-matlab');
addpath('code/ssf');
addpath('code/util');
ssf_setup;

conf = ssf_conf();

% Set config
vis = 1; % Visualize result.
overwrite=0; % Overwrite existed matching result. If false, will use existed mat result.

imgDir = 'data/SLS/';
matDir = 'output/qualitative/mat/';
figDir='output/qualitative/figure/';

for mkDir = {matDir figDir}
    if ~exist(mkDir{1}, 'dir'); mkdir(mkDir{1}); end;
end

fid=fopen('data/sls_all.txt', 'r'); l=textscan(fid, '%s'); l=l{1}; fclose(fid);
ids=l(1:3:end-2);src=l(2:3:end-1);tgt=l(3:3:end);

parfor i = 1:length(ids)
    id=ids{i}; fprintf('%s -> %s\n', src{i}, tgt{i});
    im1 = imread([imgDir src{i}]); 
	im2 = imread([imgDir tgt{i}]);
    
    fname = [matDir sprintf('%s-ssf.mat', id)];
    if overwrite || ~exist(fname, 'file') 
       [fwFlow, fwLabel, ~, fwIter] = ssf(im1, im2, conf);
       [bwFlow, bwLabel, ~, bwIter] = ssf(im2, im1, conf);
       save_caller(fname, 'fwFlow', fwFlow, 'bwFlow', bwFlow,...
       'fwError', fwIter.steps(end).error(2),'bwError', bwIter.steps(end).error(2),...
       'fwLabel', fwLabel, 'bwLabel', bwLabel,...
       'params', conf); % save parameter for reference
    end    
end

if vis % parfor can not support imshow, so keep visualization code in a seperate section 
    for i = 1:length(ids)
        id=ids{i}; fprintf('%s -> %s\n', src{i}, tgt{i});
        im1 = imread([imgDir src{i}]); 
        im2 = imread([imgDir tgt{i}]);

        fname = [matDir sprintf('%s-ssf.mat', id)];
        
        tmp = load(fname);
        fwFlow = tmp.s.fwFlow; bwFlow = tmp.s.bwFlow;
        fwError = tmp.s.fwError; bwError = tmp.s.bwError;
        fwLabel = tmp.s.fwLabel; bwLabel = tmp.s.bwLabel;

        close all;
%         [fwIm1, ~] = plot_warp(im1, im2, fwFlow, fwLabel);
%         [~, bwIm1] = plot_warp(im2, im1, bwFlow, bwLabel);
%         pause;
        
        [fwIm1, ~] = warp(im1, im2, fwFlow);
        [~, bwIm1] = warp(im2, im1, bwFlow);

        subplot(131); imshow(im1); title('Source image');
        subplot(132); imshow(im2); title('Target image');
        if fwError < bwError
            imwrite(fwIm1, [figDir id '_ssf.png']);
            subplot(133); imshow(fwIm1); title('Warped source');
        else
            imwrite(bwIm1, [figDir id '_ssf.png']);
            subplot(133); imshow(bwIm1); title('Warped source');
        end
        pause;
    end
end
% end