/*
Copyright (C) 2006 Pedro Felzenszwalb

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
*/

/* a simple image class */
// It's an extension of image.h to allow the third channel.

#ifndef IMAGE3_H
#define IMAGE3_H
#include <string.h>

template <class T>
class image3 {
 public:
  /* create an mult-channel image */
  image3(const int width, const int height, const int channel, const bool init = true);

  /* delete an image */
  ~image3();

  /* init an image */
  void init(const T &val);

  /* copy an image */
  image3<T> *copy() const;
  
  /* get the width of an image. */
  int width() const { return w; }
  
  /* get the height of an image. */
  int height() const { return h; }

  /* get the channel of an image. */
  int channel() const { return c; }
  
  /* image data. */
  T *data;
  
  /* row pointers. */
  T ***access;
  
 private:
  int w, h, c;
};

/* use imRef to access image data. */
#define imRef3(im, x, y, c) (im->access[y][x][c])
  
/* use imPtr to get pointer to image data. */
#define imPtr3(im, x, y, c) &(im->access[y][x][c])

/* use imChrPtr3 to get pointer to each pixel */ 
#define imChrPtr3(im, x, y) (im->access[y][x])

template <class T>
image3<T>::image3(const int width, const int height, const int channel, const bool init) {
  w = width;
  h = height;
  c = channel;
  data = new T[w * h * c];  // allocate space for image data
  access = new T**[h];   // allocate space for row pointers
  T **channel_ptr = new T*[w * h]; 
  
  // initialize row pointers
  for (int i = 0; i < h; i++)
	  access[i] = channel_ptr + (i * w);  

  for (int i = 0; i < w*h; i++)
	  channel_ptr[i] = data + (i * c);
  
  if (init)
	  memset(data, 0, w * h * c * sizeof(T));
}

template <class T>
image3<T>::~image3() {
	delete [] data;
	delete [] access;
}

template <class T>
void image3<T>::init(const T &val) {
  T *ptr = imPtr3(this, 0, 0, 0);
  T *end = imPtr3(this, w-1, h-1, c-1);
  while (ptr <= end)
	  *ptr++ = val;
}

template <class T>
image3<T> *image3<T>::copy() const {
  image3<T> *im = new image3<T>(w, h, c, false);
  memcpy(im->data, data, w * h * c * sizeof(T));
  return im;
}

#endif
  
