% Weichao Qiu(qiuwch@gmail.com), 2013
function [flow, scaleLabel, energylist, iter] = ssf(im1, im2, conf)
	% SSF Compute Scale-space Sift Flow
	% im1 : can be image or sift pyramid
	% im2 :
	% conf 

	% output      
	% flow       : 
	% label      : estimated scale field
	% energylist : 
	% iter       : result for each iteration

	% Debug information
    iter = struct('im1', [], 'im2', [], 'init', [], 'steps', []);
    steps = struct('label', [], 'flow', []);

    % Generate feature pyramid for dense matching.
	if isstruct(im1) && isstruct(im2)
		p1 = im1; p2 = im2; % allow input dense sift pyramid directly
	else
		p1 = ssf_multiscale_dsift(im1, conf.im1Sigmas, conf.padImage);
		p2 = ssf_multiscale_dsift(im2, conf.im2Sigmas, conf.padImage);
	end
    
    % Use single scale SiftFlow as initialization
    [scaleLabel, match] = ssf_init(p1, p2, conf.initParams, conf.scaleParams);
    init = struct('match', match, 'label', scaleLabel);
    flow = combine_flow(scaleLabel, match); 
    
    for i = 1:conf.n_iter
		% Iteratively estimate scale and flow
        sift1 = combine_sift(p1, scaleLabel);
        sift2 = p2(1).sift; 

		% Estimate flow
        [flow, foo] = refine_flow(sift1, sift2, flow, conf.refineParams);
        vx = flow(:,:,1); vy = flow(:,:,2);
        
		% Estimate label
        warpSift2 = ssf_warp(sift2, vx, vy);
        dataterm = zeros(size(vx,1), size(vx,2), length(p1));
        for j = 1:length(p1)
            siftDataTerm = sum(abs(double(warpSift2) - double(p1(j).sift)), 3);
            dataterm(:,:,j) = siftDataTerm;
        end

        scaleLabel = ssf_MRF_scale(dataterm, conf.scaleParams.beta, conf.scaleParams.tau);
        steps(i).label = scaleLabel;
        steps(i).flow = flow;
		matching_residual=sum(abs(sift1 - warpSift2), 3);
		steps(i).error = [sum(sum(matching_residual)) mean2(matching_residual)];
    end
    
    iter.im1 = im1; iter.im2 = im2;
    iter.init = init;
    iter.steps = steps;
    iter.conf = conf; % store configuration to reproduce result
    
	energylist(1).data = foo; % This energylist is for labeltransfer code
    scaleLabel = conf.im1Sigmas(scaleLabel);
end

function flow = combine_flow(scaleLabel, match)
    % combine dense matching of each scale.
    vx = match(1).vx .* 0;
    vy = match(1).vy .* 0;
    for i = 1:length(match)
        % label from 0 to n-1
        mask = (scaleLabel == i);
        vx = vx + match(i).vx .* mask;
        vy = vy + match(i).vy .* mask;
    end
    flow(:,:,1) = vx; flow(:,:,2) = vy;
end

function m_scale = combine_sift(pyramid, scaleLabel)
    sz = size(pyramid(1).sift);
    m_scale = zeros(sz(1)*sz(2), sz(3));
    for i = 1:length(pyramid);
        s_map = (scaleLabel == i);
        s_map = s_map(:);
        s_scale = pyramid(i).sift;
        s_scale = reshape(s_scale, [sz(1)*sz(2), sz(3)]);

        m_scale(s_map, :) = s_scale(s_map, :);
    end
    m_scale = reshape(m_scale, sz);
end


function [flow, foo] = refine_flow(sift1, sift2, initFlow, param)
    alpha = param.alpha;
    d = param.d;
    gamma = param.gamma;
    iteration = param.nIterations;
    wSize = param.wsize;
    n_hierarchy = 2;
    
    init_vx = initFlow(:,:,1); init_vy = initFlow(:,:,2);
    height = size(init_vx, 1); width = size(init_vy, 2);
    winSizeX=ones(height,width)*wSize;
    winSizeY=ones(height,width)*wSize;

    % [flow,foo]=ssf_mexDiscreteFlow(sift1,sift2,[alpha,d,gamma*2^(i-1),iteration,n_hierarchy,wSize],init_vx,init_vy,winSizeX,winSizeY);
    [flow,foo]=ssf_mexDiscreteFlow(sift1,sift2,[alpha,d,gamma,iteration,n_hierarchy,wSize],init_vx,init_vy,winSizeX,winSizeY);
end
