function conf = ssf_conf()
% SSF_CONF: default configuration for scale-space SIFT flow
% initParams: parameter for initialization
% refineParams: parameter for iterative refine
    conf = struct();
    
    % Setting for dense sift extraction
	% pyramid.sigmas = linspace(0.5,12,20);
    
    % default parameter for dense SIFT flow
    % from Ce Liu's SIFT Flow code
    % alpha:   (0.01) the coefficient of the truncated L1-norm regularization of the flow discontinuity
    % d:       (alpha*20) the threshold of the truncated L1-norm
    % gamma:   (0.001) the coefficient of the regularization on flow magnitude (L1 norm)
    % nlevels: (4) the number of levels of the Gaussian pyramid
    % topwsize: (10) the size of the matching window at the top level
    % nTopIterations: (100) the number of BP iterations at the top level
    % wsize:   (3) the size of the matching window at lower levels
    % nIterations: (40) the number of BP iterations at lower levels
    
    alpha = 3; 
    d = alpha * 20; 
    gamma = 0;
    initParams.alpha=alpha;
    initParams.d=d;
    initParams.gamma=gamma;
    initParams.nlevels=4;
    initParams.wsize=5;
    initParams.topwsize=20;
    initParams.nIterations=60;

    refineParams.alpha=alpha;
    refineParams.d=d;
    refineParams.gamma=gamma;
    refineParams.wsize=5;
    % RefineFlowPara.topwsize=20;
    refineParams.nIterations=60;
    
	conf.im1Sigmas=[1 2 4 6 8]; % scale of SIFT
    conf.im2Sigmas= [1]; % in current formulation, feature of im2 is extracted in original scale
	conf.padImage=1;        % make dense SIFT same size as image by expanding image border

    beta = 60;
    tau = beta * 2;
	% MRF parameters used to estimate scale field
    scaleParams.beta = beta;
    scaleParams.tau = tau;
	
	% Iteration number used to refine scale field
	conf.n_iter=3;

    conf.initParams = initParams;
    conf.refineParams = refineParams; 
    conf.scaleParams = scaleParams; % parameter for scale field
end

